--[[
Copyright (C) Achimobil, 2022

Author: Achimobil
Date: 01.06.2022
Version: 0.1.1.0

Contact:
https://forum.giants-software.com
https://discord.gg/Va7JNnEkcW(Achimobil) discord.gg/gHmnFZAypk(Revamp)

History:
V 0.1.0.0 @ 01.06.2022 - First Version.
V 0.1.0.1 @ 06.06.2022 - Changed display clipdistance from 300 to 500(Braeven)

Important:
Free for use in other mods - no permission needed, only provide my name.
No changes are to be made to this script without permission from Achimobil.

Frei verwendbar - keine erlaubnis nötig, Namensnennung im Mod erforderlich.
An diesem Skript dürfen ohne Genehmigung von Achimobil keine Änderungen vorgenommen werden.
]]

ProductionDisplaySpecialization = {
    Version = "0.1.0.0",
    Name = "ProductionDisplaySpecialization"
}
print(g_currentModName .. " - init " .. ProductionDisplaySpecialization.Name .. "(Version: " .. ProductionDisplaySpecialization.Version .. ")");

function ProductionDisplaySpecialization.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(PlaceableProductionPoint, specializations);
end

function ProductionDisplaySpecialization.registerEventListeners(placeableType)
	SpecializationUtil.registerEventListener(placeableType, "onLoad", ProductionDisplaySpecialization);
	SpecializationUtil.registerEventListener(placeableType, "onFinalizePlacement", ProductionDisplaySpecialization);
	SpecializationUtil.registerEventListener(placeableType, "onPostFinalizePlacement", ProductionDisplaySpecialization);
end

function ProductionDisplaySpecialization.registerFunctions(placeableType)
	SpecializationUtil.registerFunction(placeableType, "updateDisplays", ProductionDisplaySpecialization.updateDisplays);
end

function ProductionDisplaySpecialization.registerXMLPaths(schema, basePath)
	schema:setXMLSpecializationType("ProductionDisplay");
    
	schema:register(XMLValueType.NODE_INDEX, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#node", "Display start node");
	schema:register(XMLValueType.STRING, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#font", "Display font name");
	schema:register(XMLValueType.STRING, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#alignment", "Display text alignment");
	schema:register(XMLValueType.FLOAT, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#size", "Display text size");
	schema:register(XMLValueType.FLOAT, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#scaleX", "Display text x scale");
	schema:register(XMLValueType.FLOAT, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#scaleY", "Display text y scale");
	schema:register(XMLValueType.STRING, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#mask", "Display text mask");
	schema:register(XMLValueType.FLOAT, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#emissiveScale", "Display emissive scale");
	schema:register(XMLValueType.COLOR, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#color", "Display text color");
	schema:register(XMLValueType.COLOR, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#hiddenColor", "Display text hidden color");
	schema:register(XMLValueType.STRING, basePath .. ".productionPoint.productionDisplays.productionDisplay(?)#fillType", "Filltype name for the Display to show amount");
    
	schema:setXMLSpecializationType();
end

function ProductionDisplaySpecialization:onLoad(savegame)
    self.spec_productionPointDisplay = {};
	local spec = self.spec_productionPointDisplay;
	local xmlFile = self.xmlFile;

	spec.productionDisplays = {};
	local i = 0;

	while true do
		local productionPointDisplayKey = string.format("placeable.productionPoint.productionDisplays.productionDisplay(%d)", i);

		if not xmlFile:hasProperty(productionPointDisplayKey) then
			break;
		end
        
		local displayNode = self.xmlFile:getValue(productionPointDisplayKey .. "#node", nil, self.components, self.i3dMappings);
        
        local fontName = self.xmlFile:getValue(productionPointDisplayKey .. "#font", "DIGIT"):upper();
        local fontMaterial = g_materialManager:getFontMaterial(fontName, self.customEnvironment);
            
        local display = {};
        local alignmentStr = self.xmlFile:getValue(productionPointDisplayKey .. "#alignment", "RIGHT");
        local alignment = RenderText["ALIGN_" .. alignmentStr:upper()] or RenderText.ALIGN_RIGHT;
        local size = self.xmlFile:getValue(productionPointDisplayKey .. "#size", 0.13);
        local scaleX = self.xmlFile:getValue(productionPointDisplayKey .. "#scaleX", 1);
        local scaleY = self.xmlFile:getValue(productionPointDisplayKey .. "#scaleY", 1);
        local mask = self.xmlFile:getValue(productionPointDisplayKey .. "#mask", "000000");
        local emissiveScale = self.xmlFile:getValue(productionPointDisplayKey .. "#emissiveScale", 0.2);
        local color = self.xmlFile:getValue(productionPointDisplayKey .. "#color", {
            0.9,
            0.9,
            0.9,
            1
        }, true);
        local hiddenColor = self.xmlFile:getValue(productionPointDisplayKey .. "#hiddenColor", nil, true);
        display.displayNode = displayNode;
        display.formatStr, display.formatPrecision = string.maskToFormat(mask);
        display.fontMaterial = fontMaterial;
        display.characterLine = fontMaterial:createCharacterLine(display.displayNode, mask:len(), size, color, hiddenColor, emissiveScale, scaleX, scaleY, alignment);
        
        local fillTypeName = xmlFile:getValue(productionPointDisplayKey .. "#fillType");
        display.fillTypeId = g_fillTypeManager:getFillTypeIndexByName(fillTypeName);
        
        table.insert(spec.productionDisplays, display);

		i = i + 1;
	end
    
	function spec.fillLevelChangedCallback(fillType, delta)
        self:updateDisplays();
	end    
end

function ProductionDisplaySpecialization:onFinalizePlacement(savegame)
	local spec = self.spec_productionPointDisplay;
    self.spec_productionPoint.productionPoint.storage:addFillLevelChangedListeners(spec.fillLevelChangedCallback);
end

function ProductionDisplaySpecialization:onPostFinalizePlacement(savegame)
    self:updateDisplays();
end


function ProductionDisplaySpecialization:updateDisplays()
	local spec = self.spec_productionPointDisplay;
	local farmId = self:getOwnerFarmId();
    
	for _, display in pairs(spec.productionDisplays) do
		local fillLevel = self.spec_productionPoint.productionPoint.storage:getFillLevel(display.fillTypeId);
		local int, floatPart = math.modf(fillLevel);
		local value = string.format(display.formatStr, int, math.abs(math.floor(floatPart * 10^display.formatPrecision)))
		display.fontMaterial:updateCharacterLine(display.characterLine, value);

    for i = 1, #display.characterLine.characters do
      local charNode = display.characterLine.characters[i]
      setClipDistance(charNode, 500)
    end
	end
end