--[[
Production Revamp
Revamp Spawner Main File

Copyright (C) braeven, Achimobil, 2022

Author: braeven, Achimobil

Date: 11.06.2022
Version: 1.0.0.2

Contact/Help/Tutorials:
discord.gg/gHmnFZAypk


Changelog:
1.0.0.2 @ 11.06.2022 - Timer hinzugefügt, damit der Spawner nicht zu schnell spawnt
1.0.0.1 @ 06.06.2022 - Bugfix: Spawner-Menü zeigt keine FillTypes ohne Paletten/Ballen mehr an
1.0.0.0 @ 01.05.2022 - Initial Release


Important:.
No changes are allowed to this script without permission from Braeven.
If you want to make a production with this script, look in the documentation, discord channels for tutorials/help or download the FS22_Revamp_Productions Pack for reference
Don't copy the script into a production, load the mod as a dependency!

Es dürfen an diesem Script keine Veränderungen ohne Erlaubnis von Braeven gemacht werden.
Wenn du eine Produktion mit diesem Script bauen möchtest, lese dir die Dokumentation, die angepinnten Tutorials im Discord durch oder guck dir die FS22_Revamp_Productions an.
Nicht das Script in Produktionen kopieren, ladet den Mod über eine Dependency!

]]

RevampSpawner = {}

--Production Revamp: Integration aPalletSilo in Produktions-Trigger/ProduktionsMenü
--Production Revamp: Based on aPalletSilo V2.2.0 by Achimobil & Braeven, Modified for productions and Production Revamp



--Production Revamp: Spawnscript Aufruf hinterlegen/überschreiben
function RevampSpawner:run()
  local ownerFarmId = self.productionPoint:getOwnerFarmId()

  if ownerFarmId == AccessHandler.EVERYONE then
    self.productionPoint:buyRequest()
  else if ownerFarmId == self.mission:getFarmId() then
    --Production Revamp: Sollte kein PalettenSpawner vorhanden sein, direkt Menü anzeigen
    if self.productionPoint.palletSpawner == nil then
      self.productionPoint:openMenu()
    else
      local availableItemsInStorages = {}

        --Production Revamp: Nur anzeigen was im Lager ist und auch OUTPUT ist
      for fillTypeIndex, fillLevel in pairs (self.productionPoint.storage.fillLevels) do
        for index, value in ipairs(self.productionPoint.outputFillTypeIdsArray) do
          if value == fillTypeIndex and fillLevel > 1 then
            --BallenTypen laden falls noch nicht geladen
            if self.baleTypes == nil then
              self:LoadBaleTypes()
            end
            local currentFillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)
            
            --Production Revamp: Filltypen ohne Palette und Ballen werden nicht angezeigt
            local output = false
            if self.baleTypes[currentFillType.name] ~= nil then
              output = true
            end
            if currentFillType.palletFilename ~= nil then
              output = true
            end
            
            if output then
              if availableItemsInStorages[fillTypeIndex] == nil then
                availableItemsInStorages[fillTypeIndex] = {}
                availableItemsInStorages[fillTypeIndex].fillTypeIndex = fillTypeIndex
                availableItemsInStorages[fillTypeIndex].fillLevel = fillLevel

                availableItemsInStorages[fillTypeIndex].title = g_currentMission.fillTypeManager.fillTypes[fillTypeIndex].title
              end
            end
          end
        end
      end

      local selectableOptions = {}
      local options = {}

      --Production Revamp: Menü öffnen Option hinzufügen
      local openmenu = {}
      openmenu.openmenu = true
      if not skipmenue then
        table.insert(selectableOptions, openmenu)
        table.insert(options, g_i18n:getText("Revamp_OpenProduction"))
      end

      local empty = true
      --FillType-Liste erstellen
      for _, availableItem in pairs (availableItemsInStorages) do
        table.insert(selectableOptions, availableItem)
        table.insert(options, availableItem.title .. " (" .. math.floor(availableItem.fillLevel) .. " l)")
        empty = false
      end
      
      if empty then
        local dummy = {}
        dummy.empty = true
        table.insert(selectableOptions, dummy)
        table.insert(options, g_i18n:getText("Revamp_StorageEmpty"))
      end

      --Dialog erstellen
      local dialogArguments = {
        text = g_i18n:getText("Revamp_ChooseWhatToPutOut"),
        title = self.productionPoint.name,
        options = options,
        target = self,
        args = selectableOptions,
        callback = self.fillTypeSelected
      }

      --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
      local dialog = g_gui.guis["OptionDialog"]
      if dialog ~= nil then
        dialog.target:setOptions({""}) -- Add fake option to force a "reset"
      end

      g_gui:showOptionDialog(dialogArguments)
      end
    end
  end
end

ProductionPointActivatable.run = Utils.overwrittenFunction(ProductionPointActivatable.run, RevampSpawner.run)



--Production Revamp: Filltype ausgewählt, Mengen-Optionen anzeigen/BallenGrößen anzeigen
function ProductionPointActivatable:fillTypeSelected(selectedOption, args, rproductionPoint)
  if rproductionPoint == nil then
    rproductionPoint = self.productionPoint
  end
  local selectedArg = args[selectedOption]
  if selectedArg == nil then return end

  if selectedArg.openmenu then
    rproductionPoint:openMenu()
  elseif selectedArg.empty then

  else
    local fillTypeIndex = selectedArg.fillTypeIndex

    --Ballen Übersicht laden falls noch nicht geladen
    if self.baleTypes == nil then
      self:LoadBaleTypes()
    end  

    --Liste überprüfen ob ein Filltype in der Ballen-Liste auftaucht, sollte kein Ballen vorhanden sein, wie Palette behandeln, ansonsten Ballenliste weiter auswerten
    local currentFillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)

    if self.baleTypes[currentFillType.name] == nil then
      --Werte für Spawner hinterlegen
      rproductionPoint.pendingLiters = selectedArg.fillLevel
      rproductionPoint.fillTypeIndex = selectedArg.fillTypeIndex

      --Berechnen der maximalen Palettenanzahl
      local amountPerPallet = rproductionPoint.palletSpawner.fillTypeIdToPallet[fillTypeIndex].capacity
      local maxPallets = math.floor(selectedArg.fillLevel / amountPerPallet)
      if ((selectedArg.fillLevel - (maxPallets*amountPerPallet)) >= 1) then
        maxPallets = maxPallets + 1
      end

      if(maxPallets == 0) then return end

      --Auswählbare Palettenanzahl in Liste eintragen
      local selectableOptions = {}
      local options = {}
      for i=1, maxPallets do
        table.insert(selectableOptions, {amount=i, amountPerPallet=amountPerPallet, fillTypeIndex=fillTypeIndex, productionPoint=rproductionPoint})
        table.insert(options, i .. " " .. g_i18n:getText("Revamp_PalletSiloItem") .. " (" ..g_i18n:formatVolume(amountPerPallet * i, 0) .. ")")
      end

      --Dialog erstellen
      local dialogArguments = {
        text = g_i18n:getText("Revamp_ChooseAmountToPutOut") .. " - " .. currentFillType.title .. " (" .. g_i18n:formatVolume(selectedArg.fillLevel, 0) .. ")",
        title = rproductionPoint.name,
        options = options,
        target = self,
        args = selectableOptions,
        callback = self.amountSelected
      }

      --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
      local dialog = g_gui.guis["OptionDialog"]
      if dialog ~= nil then
        dialog.target:setOptions({""}) -- Add fake option to force a "reset"
      end
      g_gui:showOptionDialog(dialogArguments)
    else
      baleType = self.baleTypes[currentFillType.name]
      local selectableOptions = {}
      local options = {}

      --BallenVarianten in Optionsliste eintragen mit den entsprechenden Daten, Options = AnzeigeName, selectableObtion = Übermittelte Werte
      for index, baleSize in ipairs(baleType.sizes) do
        local title
        if baleSize.isRoundbale then
          title = g_i18n:getText("fillType_roundBale") .. " " .. tostring(baleSize.diameter) .. "m (" .. tostring(baleSize.capacity) .. "L)"
        else
          title = g_i18n:getText("fillType_squareBale") .. " " .. tostring(baleSize.length) .. "m (" .. tostring(baleSize.capacity) .. "L)"
        end
        table.insert(selectableOptions, {fillTypeIndex=fillTypeIndex, baleSize=baleSize, fillLevel=selectedArg.fillLevel, productionPoint=rproductionPoint})
        table.insert(options, title)
      end

      --Dialogbox erstellen welcher Ballen ausgelagert werden soll
      local dialogArguments = {
        text = g_i18n:getText("Revamp_ChooseBaleType") .. " - " .. currentFillType.title .. " (" .. g_i18n:formatVolume(selectedArg.fillLevel, 0) .. ")",
        title = rproductionPoint.name,
        options = options,
        target = self,
        args = selectableOptions,
        callback = self.baleSelected
      }

      --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
      local dialog = g_gui.guis["OptionDialog"]
      if dialog ~= nil then
        dialog.target:setOptions({""}) -- Add fake option to force a "reset"
      end
      g_gui:showOptionDialog(dialogArguments)    
    end
  end
end



--Production Revamp: Ausgewählte Anzahl Paletten spawnen
function ProductionPointActivatable:amountSelected(selectedOption, args)
  local selectedArg = args[selectedOption]
  if selectedArg == nil then return end
  local totalAmount = selectedArg.amount * selectedArg.amountPerPallet
  local productionPoint = selectedArg.productionPoint

  --Auzulagernde Menge hinterlegen
  productionPoint.pendingLiters = math.min(productionPoint.pendingLiters, totalAmount)
  --Event aufrufen für Multiplayer
  RevampSpawnPalletsEvent.sendEvent(productionPoint, productionPoint.ownerFarmId, selectedArg.fillTypeIndex, productionPoint.pendingLiters)
end



--Production Revamp: Receive function from SendEvent
function ProductionPoint:ReceivePalletEvent(ownerFarmId, fillTypeIndex, pendingLiters)
  self.pendingLiters = pendingLiters
  self.fillTypeIndex = fillTypeIndex
  self.palletSpawner:spawnPallet(ownerFarmId, fillTypeIndex, ProductionPoint.getPalletCallback, self)
end



--Production Revamp: Receive function from SendEvent
function ProductionPoint:ReceiveBaleEvent(ownerFarmId, fillTypeIndex, pendingLiters, isBale, isRoundbale, width, height, length, diameter, capacity, wrapState)
  --Werte hinterlegen für den Spawner
  self.pendingLiters = pendingLiters
  self.fillTypeIndex = fillTypeIndex

  self.palletSpawner:spawnBale(ownerFarmId, fillTypeIndex, ProductionPoint.getPalletCallback, self, isBale, isRoundbale, width, height, length, diameter, capacity, wrapState)
end



--Production Revamp: CallBack fürs Paletten/Ballen Spawnen
function ProductionPoint:getPalletCallback(pallet, result, fillTypeIndex)
  --Production Revamp: Spawner abschalten, da Palette gespawnt wurde
  self.spawnPending = false
  if pallet ~= nil then
    local delta = 0
    --Nur ausführen sollte es eine Palette sein
    if pallet.isBale == nil then
      if result == PalletSpawner.RESULT_SUCCESS then
        pallet:emptyAllFillUnits(true)
      end
      --Die noch leere Palette befüllen
      delta = pallet:addFillUnitFillLevel(self.ownerFarmId, 1, self.pendingLiters, fillTypeIndex, ToolType.UNDEFINED)
    else
      --Da Ballen mit vorgegebenen Volumen spawnen, Capacity auslesen um sie aus dem Storage zu entfernen
      delta = pallet.capacity
    end

    self.pendingLiters = math.max(self.pendingLiters - delta, 0)

    --Filllevel aus Storage abziehen
    local available = self.storage.fillLevels[fillTypeIndex]
    if available ~= null and available > 0 then
      local moved = math.min(delta, available)
      self.storage:setFillLevel(available - moved, fillTypeIndex)
    end
    if self.pendingLiters > 5 then
      --Damit das gesammt FillVolume ausgelager werden kann, überprüfen ob der Ballen größer ist als der restliche Inhalt und anpassen
      if pallet.isBale and self.pendingLiters < pallet.capacity then
        pallet.capacity = self.pendingLiters
      end
      self:updatePallets(pallet)
    end
  end
end



--Production Revamp: CallBack für den Timer
function ProductionPoint:TimerCallback()
  if self.isbale == nil then
    self.palletSpawner:spawnPallet(self.ownerFarmId, self.fillTypeIndex, ProductionPoint.getPalletCallback, self)
  else
    local bale = self.bale
    self.palletSpawner:spawnBale(self.ownerFarmId, self.fillTypeIndex, ProductionPoint.getPalletCallback, self, bale.isBale, bale.isRoundbale, bale.width, bale.height, bale.length, bale.diameter, bale.capacity, bale.wrapState)
  end
end



--Production Revamp: UpdateFunktion für Palettenspawner, Timer von 200ms um zu schnelles Spawnen zu vermeiden
function RevampSpawner:updatePallets(superFunc, bale)
  if self.isServer then
    if not self.spawnPending and self.pendingLiters > 5 then
      self.spawnPending = true
      if bale.isBale == nil then
        self.isbale = nil
        self.palletTimer = Timer.new(200)
        self.palletTimer:setFinishCallback(
            function()
                self:TimerCallback()
            end)
        self.palletTimer:start(true)
			else
        self.isbale = true
        self.bale = bale
        self.palletTimer = Timer.new(200)
        self.palletTimer:setFinishCallback(
            function()
                self:TimerCallback()
            end)
        self.palletTimer:start(true)
			end
    end
  end
end
ProductionPoint.updatePallets = Utils.overwrittenFunction(ProductionPoint.updatePallets, RevampSpawner.updatePallets)



--Production Revamp: BaleType-Liste erstellen falls noch nicht vorhanden
function ProductionPointActivatable:LoadBaleTypes()
  local baleTypes = { }

  --Ballenliste erstellen in Abhängigkeit vom Filltype, mögliche Ballen werden in Size hinterlegt als weitere Liste
  for index, baleType in ipairs(g_baleManager.bales) do

    --Sollte ein Ballen den Flag isAvaible auf false haben ist er für eine Karte deaktiviert, wie z.B. der packedsquareBale120/Multibale
    if baleType.isAvailable then
      for index, baleFillType in ipairs(baleType.fillTypes) do
        local fillType = g_fillTypeManager:getFillTypeByIndex(baleFillType.fillTypeIndex)
        local fillTypeName = fillType.name

        --BallenTypen in Abhängigkeit vom Filltype
        baleTypes[fillTypeName] = baleTypes[fillTypeName] or {
        fillTypeIndex = baleFillType.fillTypeIndex,
        fillTypeTitle = fillType.title,
        fillTypeName = fillTypeName,
        sizes = {},
      }

      local baleSizes = baleTypes[fillTypeName].sizes

      --Mögliche Ballenformate
      baleSizes[#baleSizes + 1] = {
        isRoundbale = baleType.isRoundbale,
        diameter = baleType.diameter,
        width = baleType.width,
        height = baleType.height,
        length = baleType.length,
        capacity = baleFillType.capacity,
        wrapState = true and (fillTypeName:upper() == "SILAGE")
      }
      end
    end
  end
  self.baleTypes = baleTypes
end



--Production Revamp: Ballen Variante augewählt, Mengen-Optionen anzeigen
function ProductionPointActivatable:baleSelected(selectedOption, args)
  --Parameter auslesen
  local selectedArg = args[selectedOption]
  if selectedArg == nil then return end
  local productionPoint = selectedArg.productionPoint
  local fillTypeIndex = selectedArg.fillTypeIndex
	local baleSize = selectedArg.baleSize
	local amountPerBale = baleSize.capacity

  --Werte für spawner hinterlegen
  productionPoint.fillTypeIndex = fillTypeIndex
  productionPoint.pendingLiters = selectedArg.fillLevel

  --Berechnen der maximalen Ballenanzahl
  local maxBales = math.floor(selectedArg.fillLevel / amountPerBale)
  if ((selectedArg.fillLevel - (maxBales*amountPerBale)) >= 1) then
    maxBales = maxBales + 1
  end

  if(maxBales == 0) then return end

  --Auswählbare Ballenanzahl in Liste eintragen
  local selectableOptions = {}
  local options = {}
  for i=1, maxBales do
    table.insert(selectableOptions, {amount=i, amountPerPallet=amountPerBale, fillTypeIndex=fillTypeIndex, baleSize=baleSize, productionPoint=productionPoint})
    table.insert(options, i .. " " .. g_i18n:getText("Revamp_BaleSiloItem") .. " (" ..g_i18n:formatVolume(amountPerBale*i, 0) .. ")")
  end

  local currentFillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex)

  --Dialog Optionen Anlegen
  local dialogArguments = {
    text = g_i18n:getText("Revamp_ChooseAmountToPutOut") .. " - " .. currentFillType.title .. " (" .. g_i18n:formatVolume(selectedArg.fillLevel, 0) .. ")",
    title = productionPoint.name,
    options = options,
    target = self,
    args = selectableOptions,
    callback = self.spawnBales
  }

  --TODO: hack to reset the "remembered" option (i.e. solve a bug in the game engine)
  local dialog = g_gui.guis["OptionDialog"]
  if dialog ~= nil then
    dialog.target:setOptions({""}) -- Add fake option to force a "reset"
  end

  g_gui:showOptionDialog(dialogArguments)
end



--Production Revamp: Ausgewähle Anzahl Ballen spawnen
function ProductionPointActivatable:spawnBales(selectedOption, args)
  -- Anzahl möglicher Ballen für eine neue Auswahl
  local selectedArg = args[selectedOption]
  if selectedArg == nil then return end
  local baleSize = selectedArg.baleSize
  local productionPoint = selectedArg.productionPoint

  --EntnahmeMenge berechnen
  local totalAmount = selectedArg.amount * selectedArg.baleSize.capacity

  productionPoint.pendingLiters = math.min(productionPoint.pendingLiters, totalAmount)

	--Damit das gesammt FillVolume ausgelager werden kann, überprüfen ob der Ballen größer ist als der restliche Inhalt und anpassen
  if productionPoint.pendingLiters < selectedArg.baleSize.capacity then
	  productionPoint.capacity = productionPoint.pendingLiters
	else
	  productionPoint.capacity = selectedArg.baleSize.capacity
  end
  RevampSpawnBalesEvent.sendEvent(productionPoint, productionPoint.ownerFarmId, selectedArg.fillTypeIndex, productionPoint.pendingLiters, true, baleSize.isRoundbale, baleSize.width, baleSize.height, baleSize.length, baleSize.diameter, baleSize.capacity, baleSize.wrapState)

end



--Production Revamp: Palettenspawner SpawnQueue / Update-Funktion erweitert um Ballen
function RevampSpawner:update(superFunc, dt)
	if #self.spawnQueue > 0 then
		if self.currentObjectToSpawn == nil then
			self.currentObjectToSpawn = self.spawnQueue[1]
      if self.currentObjectToSpawn.pallet.isBale==nil then
        g_currentMission.placementManager:getPlaceAsync(self.spawnPlaces, self.currentObjectToSpawn.pallet.size, self.onSpawnSearchFinished, self)
      else
			  g_currentMission.placementManager:getPlaceAsync(self.spawnPlaces, self.currentObjectToSpawn.pallet.size, self.onSpawnSearchFinishedBale, self)
      end
		end
	else
		g_currentMission:removeUpdateable(self)
	end
end
PalletSpawner.update = Utils.overwrittenFunction(PalletSpawner.update, RevampSpawner.update)



--Production Revamp: Auftrag Ballen zu generieren in der SpawnQueue hinterlegen
function PalletSpawner:spawnBale(farmId, fillTypeId, callback, callbackTarget, isBale, isRoundbale, width, height, length, diameter, capacity, wrapState)
  local pallet = nil
  if isBale then
    --Ballen Daten laden
    local baleXMLFilename = g_baleManager:getBaleXMLFilename(fillTypeId, isRoundbale, width, height, length, diameter)

    --Ballen Abmessung hinterlegen für Spawncheck
    local size = {}
    local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeId)
    size.height = height
    if isRoundbale then
      --Maße vom Rundballen hinterlegen
      size.width = diameter
      size.length = diameter
    elseif fillType.name == "COTTON" then
      --Maße vom Cotton Quaderballen drehen für den Spawnbereich, damit dieser Spawnen kann
      size.width = length
      size.length = width
    else
      --Maße von Quaderballen hinterlegen
      size.width = width
      size.length = length	    
    end

    --Fake Palette anlegen, damit der Spawner weiterhin funktioniert
    pallet = {}
    pallet.filename = baleXMLFilename
    pallet.size = size
    pallet.capacity = capacity
    pallet.isBale = true
    pallet.isRoundbale = isRoundbale
    pallet.wrapState = wrapState
    pallet.width = width
    pallet.height = height
    pallet.length = length
    pallet.diameter = diameter
  end

  if pallet ~= nil then
    table.insert(self.spawnQueue, {
      pallet = pallet,
      fillType = fillTypeId,
      farmId = farmId,
      callback = callback,
      callbackTarget = callbackTarget
    })
    g_currentMission:addUpdateable(self)
  else
    Logging.devError("RevampSpawner: no bale for fillTypeId", fillTypeId)
    callback(callbackTarget, nil, PalletSpawner.NO_PALLET_FOR_FILLTYPE, fillTypeId)
  end
end



--Production Revamp: Ballen generieren
function PalletSpawner:onSpawnSearchFinishedBale(location)
  local objectToSpawn = self.currentObjectToSpawn
  if location ~= nil then
    location.y = location.y + 0.25
    --Ballen generieren
    local baleObject = Bale.new(g_currentMission:getIsServer(), g_currentMission:getIsClient())
    if objectToSpawn.pallet.isRoundbale then
      --Rundballen auf die Seite drehen, damit diese nicht wegrollen
      location.xRot = location.xRot + (3.1415927 / 2)
    end
    local fillType = g_fillTypeManager:getFillTypeByIndex(objectToSpawn.fillType)
    if fillType.name == "COTTON" then
      --Cotton Quaderballen drehen, damit diese in den Spawnbereich passen
      location.yRot = location.yRot + (3.1415927 / 2)
      --Cotton Quaderballen zusätzliche 0,25m nach oben verschieben
      location.y = location.y + 1.30
    end
    if baleObject:loadFromConfigXML(objectToSpawn.pallet.filename, location.x, location.y, location.z, location.xRot, location.yRot, location.zRot) then
      baleObject:setFillType(objectToSpawn.fillType, true)
      baleObject:setOwnerFarmId(objectToSpawn.farmId, true)
      baleObject:setFillLevel(objectToSpawn.pallet.capacity, true)
      if objectToSpawn.pallet.wrapState then
        --SilageBallen eingewickelt Spawnen
        baleObject:setWrappingState(1)
      end
      baleObject:register()
      --Manueller Callback
      objectToSpawn.callback(objectToSpawn.callbackTarget, objectToSpawn.pallet, PalletSpawner.RESULT_SUCCESS, objectToSpawn.fillType)
      self.currentObjectToSpawn = nil
      table.remove(self.spawnQueue, 1)
    else
      print("Could not spawn bale object")
    end
  else
    objectToSpawn.callback(objectToSpawn.callbackTarget, nil, PalletSpawner.RESULT_NO_SPACE)

    self.currentObjectToSpawn = nil

    table.remove(self.spawnQueue, 1)
  end
end


--Production Revamp: Connector-Funktion zum Spawner-Script vom ProduktionsMenü(Namensräume)
function InGameMenuProductionFrame:menuconnector()
  local _, productionPoint = self:getSelectedProduction()
  local fillType, isInput = self:getSelectedStorageFillType()
  local fillLevel = self.selectedProductionPoint.storage:getFillLevel(fillType)
  
  local args = {}
  
  --Production Revamp: Fake-Eintrag damit der Rest funktioniert
  local insert = {}
  insert.fillTypeIndex = fillType
  insert.fillLevel = fillLevel
  insert.title = g_currentMission.fillTypeManager.fillTypes[fillType].title
  table.insert(args, insert)
  
  ProductionPointActivatable:fillTypeSelected(1, args, productionPoint)
end



--Production Revamp: MenüButten hinzufügen zum Auslagern
function RevampSpawner:updateMenuButtons(superFunc)
  local isProductionListActive = self.productionList == FocusManager:getFocusedElement()
  local fillType, isInput = self:getSelectedStorageFillType()

  if not isProductionListActive and fillType ~= FillType.UNKNOWN then
    if not isInput then
      local _, productionPoint = self:getSelectedProduction()
      if productionPoint.palletSpawner == nil then
      else
        --BallenTypen laden falls noch nicht geladen
        if ProductionPointActivatable.baleTypes == nil then
          ProductionPointActivatable:LoadBaleTypes()
        end
        local currentFillType = g_fillTypeManager:getFillTypeByIndex(fillType)

          --Production Revamp: Filltypen ohne Palette und Ballen werden nicht angezeigt
          local output = false
          if ProductionPointActivatable.baleTypes[currentFillType.name] ~= nil then
            output = true
          end
          if currentFillType.palletFilename ~= nil then
            output = true
          end
          
          if output then
          table.insert(self.menuButtonInfo, {
            profile = "buttonOk",
            inputAction = InputAction.MENU_ACTIVATE,
            text = self.i18n:getText("Revamp_SpawnItem"),
            callback = function()
              self:menuconnector()
            end
          })
          self:setMenuButtonInfoDirty()
        end
      end
    end
  end
end

InGameMenuProductionFrame.updateMenuButtons = Utils.appendedFunction(InGameMenuProductionFrame.updateMenuButtons, RevampSpawner.updateMenuButtons)



print("Production Revamp: Loading Pallet/Bale Spawner complete")